/***************************************************************************
 *   Copyright (C) 2007-2008 by Sebastian Kuegler <sebas@kde.org>          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/


//#include "FuncKey.h"

#include <QPainter>
#include <plasma/theme.h>
#include <kpushbutton.h>
#include "FuncKey.h"

FuncKey::FuncKey(PlasmaboardWidget *parent):
	BoardKey::BoardKey(parent) {
	// TODO Auto-generated constructor stub

	oldStyle = Plasma::PushButton::styleSheet();
	pressed = false;
}

void FuncKey::toggleOn(){
	Plasma::PushButton::setStyleSheet(oldStyle + "color:red;text-align:center;"); // sorry, but I need some visual confirmation
	pressed = true;
}

void FuncKey::toggleOff(){
	Plasma::PushButton::setStyleSheet(oldStyle);
	pressed = false;
}

void FuncKey::setKeycode(unsigned int code, bool sendUp){
	keycode = Helpers::keysymToKeycode(code);
	if(sendUp){
		QObject::connect(static_cast<const KPushButton*>(this->nativeWidget()), SIGNAL( pressed() ), this, SLOT( sendKeycodePress() ) );
		QObject::connect(static_cast<const KPushButton*>(this->nativeWidget()), SIGNAL( released() ), this, SLOT( sendKeycodeRelease() ) );
	}
	else {
		QObject::connect(static_cast<const KPushButton*>(this->nativeWidget()), SIGNAL( pressed() ), this, SLOT( sendKeycodeToggled() ) );
	}
}

void FuncKey::sendKeycodePress() {
	Helpers::fakeKeyPress(getKeycode());
	toggleOn();
}

void FuncKey::sendKeycodeToggled() {
	if( pressed ) {
		sendKeycodeRelease();
		toggleOff();
	}
	else {
		Helpers::fakeKeyPress(getKeycode());
		toggleOn();
	}
}

void FuncKey::sendKeycodeRelease() {
	Helpers::fakeKeyRelease(getKeycode());
	toggleOff();
}

void FuncKey::setUpPainter(QPainter *painter){
	painter->setRenderHint(QPainter::SmoothPixmapTransform);
	painter->setRenderHint(QPainter::Antialiasing);
	QPen pen = QPen(Plasma::Theme::defaultTheme()->color(Plasma::Theme::ButtonTextColor));
	painter->setPen(pen);
	painter->setBrush(Plasma::Theme::defaultTheme()->color(Plasma::Theme::ButtonTextColor));
}

