/***************************************************************************
 *   Copyright (C) 2007-2008 by Sebastian Kuegler <sebas@kde.org>          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include <QAction>
#include "PanelIcon.h"



PanelIcon::PanelIcon(QObject *parent, const QVariantList &args)  :
	Plasma::PopupApplet(parent, args), m_plasmaboard(0)
	{
		setAspectRatioMode(Plasma::IgnoreAspectRatio);
		setPopupIcon("preferences-desktop-keyboard");
		KConfigGroup cg = config();
		extendedMode = cg.readEntry("extendedMode", false);
}

void PanelIcon::init(){
	setPassivePopup(true);

	/*Plasma::ExtenderItem *eItem = new Plasma::ExtenderItem(extender());
	extender()->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);

	initBatteryExtender(eItem);
	QObject::connect(extender(), SIGNAL( itemAttached(Plasma::ExtenderItem *) ), this, SLOT( initBatteryExtender(Plasma::ExtenderItem *) ) );
	*/
}

void PanelIcon::toggleMode(){
	if(contextSwitcher != NULL){
		delete contextSwitcher;
	}

	KConfigGroup cg = config();

	if(!extendedMode){
		contextSwitcher = new QAction(i18n("Switch to basic mode"), this);
		if(m_plasmaboard != NULL){
			m_plasmaboard->resetKeyboard();
			m_plasmaboard->initExtendedKeyboard();
		}
		cg.writeEntry("extendedMode", true);
		extendedMode = true;
	}
	else{
		contextSwitcher = new QAction(i18n("Switch to extended mode"), this);
		if(m_plasmaboard != NULL){
			m_plasmaboard->resetKeyboard();
			m_plasmaboard->initBasicKeyboard();
		}
		cg.writeEntry("extendedMode", false);
		extendedMode = false;
	}
	connect(contextSwitcher, SIGNAL(triggered(bool)), this, SLOT(toggleMode()));
}

QGraphicsWidget *PanelIcon::graphicsWidget()
{
	if (!m_plasmaboard) {
		m_plasmaboard = new PlasmaboardWidget(this);
		contextSwitcher = new QAction(i18n("Switch to extended mode"), this);
		toggleMode();
	}
    return m_plasmaboard;
}

/*void PanelIcon::initBatteryExtender(Plasma::ExtenderItem *item){
	if (!m_plasmaboard) {
		m_plasmaboard = new PlasmaboardWidget(item);
		item->setWidget(m_plasmaboard);
		item->setTitle(i18n("Virtual Keyboard"));
	}
}*/

QList<QAction*> PanelIcon::contextualActions(){
	QList<QAction*> list;
	list.append(contextSwitcher);
	return list;
}


void PanelIcon::popupEvent(bool show){
	if ( !show ) {
		m_plasmaboard->clearAnything();
	}
}

PanelIcon::~PanelIcon() {

}


// This is the command that links your applet to the .desktop file
K_EXPORT_PLASMA_APPLET(plasmaboard, PanelIcon)
