/*/***************************************************************************
 *   Copyright (C) 2009 by Björn Ruberg <bjoern@ruberg-wegener.de>    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/


#include "widget.h"
#include "BoardKey.h"
#include "AlphaNumKey.h"
#include "FuncKey.h"
#include "EnterKey.h"
#include "BackspaceKey.h"
#include "TabKey.h"
#include "ShiftKey.h"
#include "CapsKey.h"
#include "ArrowTopKey.h"
#include "ArrowBottomKey.h"
#include "ArrowLeftKey.h"
#include "ArrowRightKey.h"
#include <QPainter>
#include <QGraphicsGridLayout>
#include <X11/Xlib.h>
#include <KConfigGroup>


#include <math.h>
#include <plasma/theme.h>

#define BACKSPACEKEY 0
#define TABKEY 1
#define ENTERKEY 2
#define CAPSKEY 3
#define SHIFT_L_KEY 4
#define SHIFT_R_KEY 5
#define CTLKEY 6
#define META_L_KEY 7
#define ALT_L_KEY 8
#define META_R_KEY 11

#define ALTGRKEY 10

#define HOMEKEY 16
#define ENDKEY 17
#define INSKEY 18
#define DELKEY 19
#define PGUPKEY 20
#define PGDOWNKEY 21

#define ARROWTOPKEY 	22
#define ARROWLEFTKEY	23
#define ARROWBOTTOMKEY	24
#define ARROWRIGHTKEY	25

PlasmaboardWidget::PlasmaboardWidget(QGraphicsWidget *parent)
    : Plasma::Applet(parent)
{

    setPreferredSize(500, 200);
    setMinimumSize(300,150);
    setCacheMode(QGraphicsItem::DeviceCoordinateCache);

    width = 500;
    height = 200;
	isLevel2 = false;
	isAlternative = false;
	isLocked = false;
	m_layout = new QGraphicsGridLayout(this);
	extendedKeys = false;
	basicKeys = false;

	//initBasicKeyboard();
}


PlasmaboardWidget::~PlasmaboardWidget(){

}

void PlasmaboardWidget::resetKeyboard(){

	int i = 0;

	if( basicKeys ){

		i = m_layout->count() - 1;
		while(i >= 0){
			m_layout->removeAt(i--);
		}

		i = 0;
		while ( i < 14 ) {
			delete funcKeys[i];
			i++;
		}

		i = 0;
		while ( i < 48 ) {
			delete alphaKeys[i];
			i++;
		}
		basicKeys = false;
	}

	i = 0;
	if( extendedKeys ) {
		while ( i < 42 ) {
			delete extKeys[i];
			i++;
		}
		extendedKeys = false;
	}
}

void PlasmaboardWidget::initExtendedKeyboard(){

	extendedKeys = true;

	int i = 0;
	while ( i < 42 ) {
		extKeys[i] = new FuncKey(this);
		QObject::connect(extKeys[i], SIGNAL( clicked() ), this, SLOT( clear() ) );
		i++;
	}

	delete extKeys[ARROWTOPKEY];
	delete extKeys[ARROWLEFTKEY];
	delete extKeys[ARROWBOTTOMKEY];
	delete extKeys[ARROWRIGHTKEY];
	extKeys[ARROWTOPKEY] = new ArrowTopKey(this);
	extKeys[ARROWLEFTKEY] = new ArrowLeftKey(this);
	extKeys[ARROWBOTTOMKEY] = new ArrowBottomKey(this);
	extKeys[ARROWRIGHTKEY] = new ArrowRightKey(this);

	m_layout->addItem(extKeys[0], 0, 0, 1, 2, Qt::AlignLeft);
	i = 1;
	for(i = 1; i < 5; i++){
		m_layout->addItem(extKeys[i], 0, (i*2)+1, 1, 2, Qt::AlignLeft);
	}
	for(i = 5; i < 9; i++){
		m_layout->addItem(extKeys[i], 0, (i*2)+2, 1, 2, Qt::AlignLeft);
	}
	for(i = 9; i < 13; i++){
		m_layout->addItem(extKeys[i], 0, (i*2)+3, 1, 2, Qt::AlignLeft);
	}

	for(i = 13; i < 16; i++){
		m_layout->addItem(extKeys[i], 0, (i*2)+4, 1, 2, Qt::AlignLeft);
	}

	extKeys[0]->setKeycode(XK_Escape, true);
	extKeys[0]->setText("ESC");
	extKeys[1]->setKeycode(XK_F1, true);
	extKeys[1]->setText("F1");
	extKeys[2]->setKeycode(XK_F2, true);
	extKeys[2]->setText("F2");
	extKeys[3]->setKeycode(XK_F3, true);
	extKeys[3]->setText("F3");
	extKeys[4]->setKeycode(XK_F4, true);
	extKeys[4]->setText("F4");
	extKeys[5]->setKeycode(XK_F5, true);
	extKeys[5]->setText("F5");
	extKeys[6]->setKeycode(XK_F6, true);
	extKeys[6]->setText("F6");
	extKeys[7]->setKeycode(XK_F7, true);
	extKeys[7]->setText("F7");
	extKeys[8]->setKeycode(XK_F8, true);
	extKeys[8]->setText("F8");
	extKeys[9]->setKeycode(XK_F9, true);
	extKeys[9]->setText("F9");
	extKeys[10]->setKeycode(XK_F10, true);
	extKeys[10]->setText("F10");
	extKeys[11]->setKeycode(XK_F11, true);
	extKeys[11]->setText("F11");
	extKeys[12]->setKeycode(XK_F12, true);
	extKeys[12]->setText("F12");
	extKeys[13]->setKeycode(XK_Print, true);
	extKeys[13]->setText("Print");
	extKeys[14]->setKeycode(XK_Num_Lock, true);
	extKeys[14]->setText("Num");
	extKeys[15]->setKeycode(XK_Pause, true);
	extKeys[15]->setText("Pause");

	initBasicKeyboard(1);


	m_layout->addItem(extKeys[ARROWLEFTKEY], 5, 29, 1, 2, Qt::AlignLeft);

	int col = 31;

	m_layout->addItem(extKeys[HOMEKEY], 1, col, 1, 2, Qt::AlignLeft);
	extKeys[HOMEKEY]->setKeycode(XK_Home, true);
	extKeys[HOMEKEY]->setText("Home");

	m_layout->addItem(extKeys[ENDKEY], 2, col, 1, 2, Qt::AlignLeft);
	extKeys[ENDKEY]->setKeycode(XK_End, true);
	extKeys[ENDKEY]->setText("End");

	m_layout->addItem(extKeys[INSKEY], 3, col, 1, 2, Qt::AlignLeft);
	extKeys[INSKEY]->setKeycode(XK_Insert, true);
	extKeys[INSKEY]->setText("Ins");

	m_layout->addItem(extKeys[ARROWTOPKEY], 4, col, 1, 2, Qt::AlignLeft);
	m_layout->addItem(extKeys[ARROWBOTTOMKEY], 5, col, 1, 2, Qt::AlignLeft);


	col+=2;

	m_layout->addItem(extKeys[ARROWRIGHTKEY], 5, col, 1, 2, Qt::AlignLeft);
	m_layout->addItem(extKeys[DELKEY], 1, col, 1, 2, Qt::AlignLeft);
	extKeys[DELKEY]->setKeycode(XK_Delete, true);
	extKeys[DELKEY]->setText("Del");
	m_layout->addItem(extKeys[PGUPKEY], 2, col, 1, 2, Qt::AlignLeft);
	extKeys[PGUPKEY]->setKeycode(XK_Page_Up, true);
	extKeys[PGUPKEY]->setText("PgUp");
	m_layout->addItem(extKeys[PGDOWNKEY], 3, col, 1, 2, Qt::AlignLeft);
	extKeys[PGDOWNKEY]->setKeycode(XK_Page_Down, true);
	extKeys[PGDOWNKEY]->setText("PgDw");

	col+=3;

	m_layout->addItem(extKeys[26], 2, col, 1, 2);
	extKeys[26]->setKeycode(XK_KP_7, true);
	extKeys[26]->setText("7");
	m_layout->addItem(extKeys[27], 3, col, 1, 2);
	extKeys[27]->setKeycode(XK_KP_4, true);
	extKeys[27]->setText("4");
	m_layout->addItem(extKeys[28], 4, col, 1, 2);
	extKeys[28]->setKeycode(XK_KP_1, true);
	extKeys[28]->setText("1");
	m_layout->addItem(extKeys[29], 5, col, 1, 4);
	extKeys[29]->setKeycode(XK_KP_0, true);
	extKeys[29]->setText("0");

	col+=2;

	m_layout->addItem(extKeys[30], 1, col, 1, 2);
	extKeys[30]->setKeycode(XK_KP_Divide, true);
	extKeys[30]->setText("/");
	m_layout->addItem(extKeys[31], 2, col, 1, 2);
	extKeys[31]->setKeycode(XK_KP_8, true);
	extKeys[31]->setText("8");
	m_layout->addItem(extKeys[32], 3, col, 1, 2);
	extKeys[32]->setKeycode(XK_KP_5, true);
	extKeys[32]->setText("5");
	m_layout->addItem(extKeys[33], 4, col, 1, 2);
	extKeys[33]->setKeycode(XK_KP_2, true);
	extKeys[33]->setText("2");

	col+=2;

	m_layout->addItem(extKeys[34], 1, col, 1, 2);
	extKeys[34]->setKeycode(XK_KP_Multiply, true);
	extKeys[34]->setText("*");
	m_layout->addItem(extKeys[35], 2, col, 1, 2);
	extKeys[35]->setKeycode(XK_KP_9, true);
	extKeys[35]->setText("9");
	m_layout->addItem(extKeys[36], 3, col, 1, 2);
	extKeys[36]->setKeycode(XK_KP_6, true);
	extKeys[36]->setText("6");
	m_layout->addItem(extKeys[37], 4, col, 1, 2);
	extKeys[37]->setKeycode(XK_KP_3, true);
	extKeys[37]->setText("3");
	m_layout->addItem(extKeys[38], 5, col, 1, 2);
	extKeys[38]->setKeycode(XK_KP_Separator, true);
	extKeys[38]->setText(".");

	col+=2;

	m_layout->addItem(extKeys[39], 1, col, 1, 2);
	extKeys[39]->setKeycode(XK_KP_Subtract, true);
	extKeys[39]->setText("-");
	m_layout->addItem(extKeys[40], 2, col, 2, 2);
	extKeys[40]->setKeycode(XK_KP_Add, true);
	extKeys[40]->setText("+");
	m_layout->addItem(extKeys[41], 4, col, 2, 2);
	extKeys[41]->setKeycode(XK_KP_Enter, true);
	extKeys[41]->setText("Enter");

	for(int i = 30; i < 44; i++){
		m_layout->setColumnPreferredWidth(i,width/30);
	}
}

void PlasmaboardWidget::initBasicKeyboard(int offset){

	basicKeys = true;

	// create objects
	int i = 6;
	while ( i < 14 ) {
		funcKeys[i] = new FuncKey(this);
		i++;
	}

	funcKeys[BACKSPACEKEY] = new BackspaceKey(this);
	funcKeys[TABKEY] = new TabKey(this);
	funcKeys[ENTERKEY] = new EnterKey(this);
	funcKeys[CAPSKEY] = new CapsKey(this);
	funcKeys[SHIFT_L_KEY] = new ShiftKey(this);
	funcKeys[SHIFT_R_KEY] = new ShiftKey(this);

	i = 0;
	while ( i < 48 ) {
		alphaKeys[i] = new AlphaNumKey(this);
		QObject::connect(alphaKeys[i], SIGNAL( clicked() ), this, SLOT( clear() ) );
		i++;
	}

	// create layout

	int numberOfCols = 30;

	int x;
	int row = offset;
	for(x = 0; x<=12; x++){
		m_layout->addItem(alphaKeys[x], row, x*2, 1, 2, Qt::AlignLeft);
	}
	m_layout->addItem(funcKeys[BACKSPACEKEY], row, x*2, 1, 4, Qt::AlignLeft);

	row++;

	m_layout->addItem(funcKeys[TABKEY], row, 0, 1, 3, Qt::AlignLeft);
	int t = 3;
	for(x = 13; x<=24; x++){
		m_layout->addItem(alphaKeys[x], row, t, 1, 2, Qt::AlignLeft);
		t+=2;
	}
	m_layout->addItem(funcKeys[ENTERKEY], row, t+1, 2, 2, Qt::AlignLeft);

	row++;

	m_layout->addItem(funcKeys[CAPSKEY], row, 0, 1, 4, Qt::AlignLeft);
	t = 4;
	for(x = 25; x<=36; x++){
		m_layout->addItem(alphaKeys[x], row, t, 1, 2, Qt::AlignLeft);
		t+=2;
	}

	row++;

	m_layout->addItem(funcKeys[SHIFT_L_KEY], row, 0, 1, 3, Qt::AlignLeft);
	t = 3;
	for(x = 37; x<=47; x++){
		m_layout->addItem(alphaKeys[x], row, t, 1, 2, Qt::AlignLeft);
		t+=2;
	}
	m_layout->addItem(funcKeys[SHIFT_R_KEY], row, t, 1, 5, Qt::AlignLeft);

	row++;

	m_layout->addItem(funcKeys[CTLKEY], row, 0, 1, 2, Qt::AlignLeft);
	m_layout->addItem(funcKeys[META_L_KEY], row, 2, 1, 2, Qt::AlignLeft);
	m_layout->addItem(funcKeys[ALT_L_KEY], row, 4, 1, 2, Qt::AlignLeft);
	m_layout->addItem(funcKeys[9], row, 6, 1, 14, Qt::AlignLeft);
	m_layout->addItem(funcKeys[ALTGRKEY], row, 20, 1, 2, Qt::AlignLeft);
	m_layout->addItem(funcKeys[META_R_KEY], row, 22, 1, 2, Qt::AlignLeft);
	m_layout->addItem(funcKeys[12], row, 24, 1, 2, Qt::AlignLeft);
	m_layout->addItem(funcKeys[13], row, 26, 1, 2, Qt::AlignLeft);

	for(int i = 0; i<numberOfCols; i++){
		m_layout->setColumnPreferredWidth(i,width/numberOfCols);
	}

	for(int i = 30; i < 44; i++){
		m_layout->setColumnPreferredWidth(i, 0);
	}

	m_layout->setSpacing(0);
	m_layout->setContentsMargins(0,0,0,0);

	// set Keymap

	alphaKeys[0]->setKeycode(49);
	alphaKeys[1]->setKeycode(10);
	alphaKeys[2]->setKeycode(11);
	alphaKeys[3]->setKeycode(12);
	alphaKeys[4]->setKeycode(13);
	alphaKeys[5]->setKeycode(14);
	alphaKeys[6]->setKeycode(15);
	alphaKeys[7]->setKeycode(16);
	alphaKeys[8]->setKeycode(17);
	alphaKeys[9]->setKeycode(18);
	alphaKeys[10]->setKeycode(19);
	alphaKeys[11]->setKeycode(20);
	alphaKeys[12]->setKeycode(21);

	alphaKeys[13]->setKeycode(24);
	alphaKeys[14]->setKeycode(25);
	alphaKeys[15]->setKeycode(26);
	alphaKeys[16]->setKeycode(27);
	alphaKeys[17]->setKeycode(28);
	alphaKeys[18]->setKeycode(29);
	alphaKeys[19]->setKeycode(30);
	alphaKeys[20]->setKeycode(31);
	alphaKeys[21]->setKeycode(32);
	alphaKeys[22]->setKeycode(33);
	alphaKeys[23]->setKeycode(34);
	alphaKeys[24]->setKeycode(35);

	alphaKeys[25]->setKeycode(38);
	alphaKeys[26]->setKeycode(39);
	alphaKeys[27]->setKeycode(40);
	alphaKeys[28]->setKeycode(41);
	alphaKeys[29]->setKeycode(42);
	alphaKeys[30]->setKeycode(43);
	alphaKeys[31]->setKeycode(44);
	alphaKeys[32]->setKeycode(45);
	alphaKeys[33]->setKeycode(46);
	alphaKeys[34]->setKeycode(47);
	alphaKeys[35]->setKeycode(48);
	alphaKeys[36]->setKeycode(51);

	alphaKeys[37]->setKeycode(94);
	alphaKeys[38]->setKeycode(52);
	alphaKeys[39]->setKeycode(53);
	alphaKeys[40]->setKeycode(54);
	alphaKeys[41]->setKeycode(55);
	alphaKeys[42]->setKeycode(56);
	alphaKeys[43]->setKeycode(57);
	alphaKeys[44]->setKeycode(58);
	alphaKeys[45]->setKeycode(59);
	alphaKeys[46]->setKeycode(60);
	alphaKeys[47]->setKeycode(61);

	funcKeys[CTLKEY]->setKeycode(XK_Control_L, false);
	funcKeys[CTLKEY]->setText("CTL");
	funcKeys[META_L_KEY]->setKeycode(XK_Meta_L, false);
	funcKeys[META_L_KEY]->setText("META");
	funcKeys[ALT_L_KEY]->setKeycode(XK_Alt_L, false);
	funcKeys[ALT_L_KEY]->setText("ALT");
	funcKeys[9]->setKeycode(XK_space, true);
	funcKeys[ALTGRKEY]->setKeycode(XK_ISO_Level3_Shift, false);
	funcKeys[ALTGRKEY]->setText("ALT GR");
	funcKeys[META_R_KEY]->setKeycode(XK_Meta_L, false);
	funcKeys[META_R_KEY]->setText("META");
	funcKeys[12]->setKeycode(XK_Menu, true);
	funcKeys[12]->setText("MENU");
	funcKeys[13]->setKeycode(XK_Control_L, false);
	funcKeys[13]->setText("CTL");

	QObject::connect(funcKeys[SHIFT_L_KEY], SIGNAL( clicked() ), this, SLOT( switchLevel() ) );
	QObject::connect(funcKeys[SHIFT_R_KEY], SIGNAL( clicked() ), this, SLOT( switchLevel() ) );
	QObject::connect(funcKeys[CAPSKEY], SIGNAL( clicked() ), this, SLOT( switchCaps() ) );
	QObject::connect(funcKeys[ALTGRKEY], SIGNAL( clicked() ), this, SLOT( switchAlternative() ) );

}

void PlasmaboardWidget::paint(QPainter *p,
                const QStyleOptionGraphicsItem *option,
                 QWidget*)
{
    p->setRenderHint(QPainter::SmoothPixmapTransform);
    p->setRenderHint(QPainter::Antialiasing);

    p->restore();
}

void PlasmaboardWidget::switchLevel(){
	if(!isLevel2){
		isLevel2 = true;
	}
	else {
		isLevel2 = false;
		Helpers::fakeKeyRelease(Helpers::keysymToKeycode(XK_Shift_L));
		funcKeys[SHIFT_L_KEY]->toggleOff();
		funcKeys[SHIFT_R_KEY]->toggleOff();
	}
	int i = 0;
	while ( i < 48 ) {
		alphaKeys[i]->switchKey(isLevel2, isAlternative, isLocked);
		i++;
	}
}

void PlasmaboardWidget::switchAlternative(){
	if(!isAlternative){
		isAlternative = true;
	}
	else {
		isAlternative = false;
		Helpers::fakeKeyRelease(Helpers::keysymToKeycode(XK_ISO_Level3_Shift));
		funcKeys[ALTGRKEY]->toggleOff();
	}
	int i = 0;
	while ( i < 48 ) {
		alphaKeys[i]->switchKey(isLevel2, isAlternative, isLocked);
		i++;
	}
}

void PlasmaboardWidget::switchCaps(){
	if(isLocked){
		isLocked = false;
	}
	else {
		isLocked = true;
	}
	int i = 0;
	while ( i < 48 ) {
		alphaKeys[i]->switchKey(isLevel2, isAlternative, isLocked);
		i++;
	}
}

void PlasmaboardWidget::clearAnything(){
	if ( isLocked ) {
		switchCaps();
		funcKeys[CAPSKEY]->sendKeycodePress();
		funcKeys[CAPSKEY]->sendKeycodeRelease();
	}
	clear();
}

void PlasmaboardWidget::clear(){
	bool change = false;
	if(isLevel2){
		Helpers::fakeKeyRelease(Helpers::keysymToKeycode(XK_Shift_L));
		funcKeys[SHIFT_L_KEY]->toggleOff();
		funcKeys[SHIFT_R_KEY]->toggleOff();
		isLevel2 = false;
		change = true;
	}
	if(isAlternative){
		Helpers::fakeKeyRelease(Helpers::keysymToKeycode(XK_ISO_Level3_Shift));
		funcKeys[ALTGRKEY]->toggleOff();
		isAlternative = false;
		change = true;
	}

	if(change){
		int i = 0;
		while ( i < 48 ) {
			alphaKeys[i]->switchKey(isLevel2, isAlternative, isLocked);
			i++;
		}
	}

	Helpers::fakeKeyRelease(Helpers::keysymToKeycode(XK_Control_L));
	funcKeys[CTLKEY]->toggleOff();
	funcKeys[13]->toggleOff();
	Helpers::fakeKeyRelease(Helpers::keysymToKeycode(XK_Meta_L));
	funcKeys[META_L_KEY]->toggleOff();
	funcKeys[META_R_KEY]->toggleOff();
	Helpers::fakeKeyRelease(Helpers::keysymToKeycode(XK_Meta_L));
	funcKeys[ALT_L_KEY]->toggleOff();
}


