/***************************************************************************
 *   Copyright (C) 2007-2008 by Sebastian Kuegler <sebas@kde.org>          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/


#ifndef WIDGET_H_
#define WIDGET_H_

#include "BoardKey.h"
#include <plasma/applet.h>
#include <plasma/svg.h>
#include "Helpers.h"

#define XK_TECHNICAL
#define XK_PUBLISHING
#define XK_LATIN1
#include <X11/keysym.h>

class AlphaNumKey;
class FuncKey;
class QChar;
class QGraphicsGridLayout;

// Define our plasma Applet
class PlasmaboardWidget : public Plasma::Applet
{
    Q_OBJECT
    public:
        // Basic Create/Destroy
    	PlasmaboardWidget(QGraphicsWidget *parent);
        ~PlasmaboardWidget();

        // The paintInterface procedure paints the applet to screen
        void paint(QPainter *painter,
                const QStyleOptionGraphicsItem *option,
                 QWidget*);
        void initBasicKeyboard(int offset=0);
        void initExtendedKeyboard();
        void resetKeyboard();

        void clearAnything();

    public Q_SLOTS:
        void switchLevel();
        void switchAlternative();
        void switchCaps();
        void clear();

	private:
		AlphaNumKey *alphaKeys[48]; // normal keys labeled with symbols like a, b, c
		FuncKey *funcKeys[14]; // functional keys like shift, backspace, enter
		FuncKey *extKeys[42]; // keys only shown in the extended layout as F1, F2,..
		bool isLevel2;
		bool isAlternative;
		bool isLocked;
		bool basicKeys;
		bool extendedKeys;
		QGraphicsGridLayout *m_layout;

		int width;
		int height;
};


#endif /* WIDGET_H_ */
